from django.core.exceptions import ObjectDoesNotExist
from edc_appointment.models import Appointment
from edc_constants.constants import NO, YES
from edc_lab.constants import TUBE
from edc_lab.model_mixins import RequisitionModelMixin
from edc_utils import get_utcnow


class RequisitionVerifier:

    """Update a requisition's "verify" fields.

    Also sets defaults for any other fields that are required
    if pre-print a label.
    """

    def __init__(self, appointment_pk=None, requisition_identifier=None):
        self._requisition = None
        self.verified = None
        self.appointment = Appointment.objects.get(pk=appointment_pk)
        self.requisition_identifier = requisition_identifier
        if self.requisition and self.requisition.is_drawn != NO:
            # verification fields
            self.requisition.clinic_verified = YES
            self.requisition.clinic_verified_datetime = get_utcnow()
            # other fields for label printing
            self.requisition.is_drawn = self.requisition.is_drawn or YES
            self.requisition.item_count = self.requisition.item_count or 1
            self.requisition.item_type = self.requisition.item_type or TUBE
            self.requisition.drawn_datetime = self.requisition.drawn_datetime or get_utcnow()
            self.requisition.save()
            self.verified = self.requisition.clinic_verified

    def __str__(self):
        return f"{self.requisition_identifier} {self.verified}"

    def __repr__(self):
        return f"<{self.__class__.__name__}(" "{self.requisition_identifier}) {self.verified}>"

    @property
    def requisition(self):
        """Returns a requisition model instance."""
        if not self._requisition:
            try:
                self._requisition = self.requisition_model_cls.objects.get(
                    requisition_identifier=self.requisition_identifier.strip()
                )
            except ObjectDoesNotExist:
                pass
        return self._requisition

    @property
    def requisition_model_cls(self):
        """Returns the requisition model class.

        Discovers the requisition model class from the visit
        model.
        """
        model_cls = None
        visit_model_cls = self.appointment.visit.__class__
        for attr in dir(visit_model_cls):
            try:
                obj = getattr(getattr(visit_model_cls, attr), "rel")
            except AttributeError:
                pass
            else:
                try:
                    model_cls = obj.related_model
                except AttributeError:
                    pass
                else:
                    if issubclass(model_cls, (RequisitionModelMixin,)):
                        break
        return model_cls



from django.core.exceptions import ImproperlyConfigured
from django.views.generic.base import ContextMixin
from edc_action_item.view_mixins import ActionItemViewMixin
from edc_appointment.view_mixins import AppointmentViewMixin
from edc_consent.view_mixins import ConsentViewMixin
from edc_dashboard.view_mixins import EdcViewMixin
from edc_dashboard.views import DashboardView
from edc_data_manager.view_mixins import DataManagerViewMixin
from edc_locator.view_mixins import SubjectLocatorViewMixin
from edc_metadata.view_mixins import MetadataViewMixin, RefreshMetadataViewMixin
from edc_navbar.view_mixin import NavbarViewMixin
from edc_subject_model_wrappers import (
    AppointmentModelWrapper,
    SubjectConsentModelWrapper,
    SubjectLocatorModelWrapper,
    SubjectVisitModelWrapper,
)
from edc_visit_schedule.view_mixins import VisitScheduleViewMixin

from ..view_mixins import RegisteredSubjectViewMixin, SubjectVisitViewMixin


class VerifyRequisitionMixin(ContextMixin):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        scanning = self.kwargs.get("scanning")
        context.update(scanning=scanning)
        return context


class SubjectDashboardView(
    EdcViewMixin,
    NavbarViewMixin,
    MetadataViewMixin,
    RefreshMetadataViewMixin,
    ConsentViewMixin,
    SubjectLocatorViewMixin,
    AppointmentViewMixin,
    ActionItemViewMixin,
    DataManagerViewMixin,
    SubjectVisitViewMixin,
    VisitScheduleViewMixin,
    RegisteredSubjectViewMixin,
    VerifyRequisitionMixin,
    DashboardView,
):

    navbar_selected_item = "consented_subject"

    dashboard_url_name = "subject_dashboard_url"
    dashboard_template = "subject_dashboard_template"

    appointment_model = "edc_appointment.appointment"
    appointment_model_wrapper_cls = AppointmentModelWrapper

    consent_model = None
    consent_model_wrapper_cls = SubjectConsentModelWrapper

    subject_locator_model = "edc_locator.subjectlocator"
    subject_locator_model_wrapper_cls = SubjectLocatorModelWrapper

    visit_model = None
    visit_model_wrapper_cls = SubjectVisitModelWrapper

    def __init__(self, **kwargs):
        if not self.navbar_name:
            raise ImproperlyConfigured(f"'navbar_name' cannot be None. See {repr(self)}.")
        self.appointment_model_wrapper_cls.visit_model_wrapper_cls = (
            self.visit_model_wrapper_cls
        )
        if self.visit_model:
            self.visit_model_wrapper_cls.model = self.visit_model
        else:
            self.visit_model = self.visit_model_wrapper_cls.model
        if self.consent_model:
            self.consent_model_wrapper_cls.model = self.consent_model
        else:
            self.consent_model = self.consent_model_wrapper_cls.model
        super().__init__(**kwargs)



from django.contrib.admin import AdminSite as DjangoAdminSite
from edc_locator.models import SubjectLocator

from .models import Appointment, SubjectConsent, SubjectVisit


class AdminSite(DjangoAdminSite):
    pass


edc_subject_dashboard_admin = AdminSite(name="edc_subject_dashboard_admin")

edc_subject_dashboard_admin.register(Appointment)
edc_subject_dashboard_admin.register(SubjectConsent)
edc_subject_dashboard_admin.register(SubjectLocator)
edc_subject_dashboard_admin.register(SubjectVisit)



from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from edc_constants.constants import NO
from edc_lab.labels import RequisitionLabel


class RequisitionLabels:

    label_cls = RequisitionLabel
    label_template_name = "requisition"

    def __init__(
        self, requisition_metadata=None, panel_names=None, appointment=None, user=None
    ):
        zpl_datas = []
        self.requisitions_not_printed = []
        self.appointment = appointment
        for metadata in requisition_metadata.filter(panel_name__in=panel_names):
            panel = [
                r
                for r in appointment.visit.visit.all_requisitions
                if r.panel.name == metadata.panel_name
            ][0].panel
            requisition = self.get_or_create_requisition(panel=panel, user=user)
            if requisition.is_drawn != NO:
                item_count = requisition.item_count or 1
                for item in range(1, item_count + 1):
                    label = self.label_cls(requisition=requisition, user=user, item=item)
                    zpl_datas.append(label.render_as_zpl_data(copies=1))
            else:
                self.requisitions_not_printed.append(requisition)
        self.zpl_data = b"".join(zpl_datas)

    def get_or_create_requisition(self, panel=None, user=None):
        """Gets or creates a requisition.

        If created, the requisition created is incomplete; that is,
        is_drawn and drawn_datetime are None.
        """
        requisition_model_cls = django_apps.get_model(panel.requisition_model)
        visit_model_attr = requisition_model_cls.visit_model_attr()
        try:
            requisition_model_obj = requisition_model_cls.objects.get(
                panel=panel.panel_model_obj,
                **{f"{visit_model_attr}__appointment": self.appointment},
            )
        except ObjectDoesNotExist:
            requisition_model_obj = requisition_model_cls(
                user_created=user.username,
                panel=panel.panel_model_obj,
                **{visit_model_attr: self.appointment.visit},
            )
            requisition_model_obj.get_requisition_identifier()
            requisition_model_obj.save()
        return requisition_model_obj



from collections import namedtuple

from dateutil.relativedelta import relativedelta
from django import template
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.utils.html import format_html
from django.utils.translation import gettext as _
from edc_appointment.constants import (
    CANCELLED_APPT,
    COMPLETE_APPT,
    IN_PROGRESS_APPT,
    INCOMPLETE_APPT,
    NEW_APPT,
)
from edc_appointment.models import Appointment
from edc_constants.constants import COMPLETE
from edc_dashboard.utils import get_bootstrap_version
from edc_lab.models.manifest.consignee import Consignee
from edc_metadata import KEYED, REQUIRED
from edc_metadata.metadata_helper import MetadataHelper
from edc_utils import get_utcnow

register = template.Library()


class SubjectDashboardExtrasError(Exception):
    pass


@register.inclusion_tag(
    f"edc_subject_dashboard/bootstrap{get_bootstrap_version()}/forms_button.html"
)
def forms_button(wrapper=None):
    """wrapper is an AppointmentModelWrapper."""

    try:
        visit_pk = wrapper.wrapped_visit.object.id
    except AttributeError:
        visit_pk = None
    if visit_pk and wrapper.appt_status == IN_PROGRESS_APPT:
        btn_color = "btn-primary"
        title = ""
        fa_icon = "fas fa-list-alt"
        href = wrapper.forms_url
        label = _("Forms")
        label_fa_icon = "fas fa-share"
        visit_pk = str(visit_pk)
    else:
        btn_color = "btn-warning"
        title = _("Click to update the visit report")
        fa_icon = "fas fa-plus"
        href = wrapper.wrapped_visit.href
        label = _("Start visit")
        label_fa_icon = None
    btn_id = f"{label.lower()}_btn_{wrapper.visit_code}_{wrapper.visit_code_sequence}"
    return dict(
        btn_color=btn_color,
        btn_id=btn_id,
        fa_icon=fa_icon,
        href=href,
        label=label,
        label_fa_icon=label_fa_icon,
        title=title,
        visit_code=wrapper.visit_code,
        visit_code_sequence=wrapper.visit_code_sequence,
        visit_pk=visit_pk,
        document_status=wrapper.wrapped_visit.object.document_status,
        COMPLETE=COMPLETE,
    )


@register.inclusion_tag(
    f"edc_subject_dashboard/bootstrap{get_bootstrap_version()}/appointment_in_progress.html"
)
def appointment_in_progress(subject_identifier=None, visit_schedule=None, schedule=None):

    try:
        appointment = Appointment.objects.get(
            subject_identifier=subject_identifier,
            visit_schedule_name=visit_schedule.name,
            schedule_name=schedule.name,
            appt_status=IN_PROGRESS_APPT,
        )
    except ObjectDoesNotExist:
        visit_code = None
    except MultipleObjectsReturned:
        qs = Appointment.objects.filter(
            subject_identifier=subject_identifier,
            visit_schedule_name=visit_schedule.name,
            schedule_name=schedule.name,
            appt_status=IN_PROGRESS_APPT,
        )
        visit_code = ", ".join([obj.visit_code for obj in qs])
    else:
        visit_code = appointment.visit_code
    return dict(visit_code=visit_code)


@register.inclusion_tag(
    f"edc_subject_dashboard/bootstrap{get_bootstrap_version()}/"
    f"requisition_panel_actions.html",
    takes_context=True,
)
def requisition_panel_actions(context, requisitions=None):
    try:
        requisition_metadata = requisitions[0]
    except IndexError:
        context["verify_disabled"] = None
    else:
        app_label, model_name = requisition_metadata.model.split(".")
        context["verify_disabled"] = (
            None
            if context["user"].has_perm(f"{app_label}.change_{model_name}")
            else "disabled"
        )
    appointment = context.get("appointment")
    scanning = context.get("scanning")
    autofocus = "autofocus" if scanning else ""
    context["appointment"] = str(appointment.object.pk)
    context["autofocus"] = autofocus
    return context


@register.inclusion_tag(
    f"edc_subject_dashboard/bootstrap{get_bootstrap_version()}/"
    f"print_requisition_popover.html",
    takes_context=True,
)
def print_requisition_popover(context):
    C = namedtuple("Consignee", "pk name")
    consignees = []
    for consignee in Consignee.objects.all():
        consignees.append(C(str(consignee.pk), consignee.name))
    context["consignees"] = consignees
    return context


@register.inclusion_tag(
    f"edc_subject_dashboard/bootstrap{get_bootstrap_version()}/" f"appointment_status.html"
)
def appointment_status_icon(appt_status=None):
    return dict(
        appt_status=appt_status,
        NEW_APPT=NEW_APPT,
        IN_PROGRESS_APPT=IN_PROGRESS_APPT,
        INCOMPLETE_APPT=INCOMPLETE_APPT,
        COMPLETE_APPT=COMPLETE_APPT,
        CANCELLED_APPT=CANCELLED_APPT,
    )


@register.inclusion_tag(
    f"edc_subject_dashboard/bootstrap{get_bootstrap_version()}/dashboard/crf_totals.html"
)
def show_crf_totals(wrapped_appointment=None, request=None):
    helper = MetadataHelper(wrapped_appointment.object)
    crf_keyed = helper.get_crf_metadata_by(entry_status=KEYED).count()
    requisition_keyed = helper.get_requisition_metadata_by(entry_status=KEYED).count()
    crf_total = helper.get_crf_metadata_by(entry_status=[REQUIRED, KEYED]).count()
    requisition_total = helper.get_requisition_metadata_by(
        entry_status=[REQUIRED, KEYED]
    ).count()
    keyed = crf_keyed + requisition_keyed
    total = crf_total + requisition_total
    if wrapped_appointment.object.appt_datetime > get_utcnow():
        show_totals = False
    else:
        show_totals = False if keyed != 0 and keyed == total else True
    return dict(
        show_totals=show_totals,
        complete=keyed != 0 and keyed == total,
        request=request,
        keyed=keyed,
        total=total,
    )


@register.inclusion_tag(
    f"edc_subject_dashboard/bootstrap{get_bootstrap_version()}/dashboard/visit_button.html"
)
def show_dashboard_visit_button(wrapped_appointment=None, request=None):

    title = None
    label = None
    btn_class = None
    if wrapped_appointment.appt_status == NEW_APPT:
        label = _("Start")
        title = _("Start data collection for this timepoint.")
        if wrapped_appointment.object.appt_datetime <= get_utcnow():
            btn_class = "warning"
    elif wrapped_appointment.appt_status == INCOMPLETE_APPT:
        incomplete = _("Incomplete")
        label = format_html(
            '<i class="fas fa-pencil-alt fa-sm" aria-hidden="true"></i> {}', incomplete
        )
        title = _("Continue data collection for this timepoint.")
    elif wrapped_appointment.appt_status == CANCELLED_APPT:
        label = _("Cancelled")
        title = _("Cancelled.")
    elif wrapped_appointment.appt_status == COMPLETE_APPT:
        # this appt_status is handled by the subject visit button
        label = _("Done")
        label = format_html(
            '<i class="fas fa-pencil-alt fa-sm" aria-hidden="true"></i> {}', label
        )
        title = _("Done. Timepoint is complete")
        btn_class = "success"

    elif wrapped_appointment.appt_status == IN_PROGRESS_APPT:
        # this appt_status is handled by the subject visit button
        pass
    else:
        raise SubjectDashboardExtrasError(
            f"Unhandled appt_status. Got {wrapped_appointment.appt_status}"
        )
    return dict(
        wrapped_appointment=wrapped_appointment,
        title=title,
        label=label,
        btn_class=btn_class,
        request=request,
    )


@register.inclusion_tag(
    f"edc_subject_dashboard/bootstrap{get_bootstrap_version()}/"
    "dashboard/appointment_button.html"
)
def show_dashboard_appointment_button(
    wrapped_appointment=None, view_appointment=None, request=None
):
    anchor_id = (
        f"appointment_btn_{wrapped_appointment.visit_code }_"
        f"{ wrapped_appointment.visit_code_sequence }"
    )
    if (
        view_appointment
        and wrapped_appointment.object.site.id == request.site.id
        and wrapped_appointment.appt_status == IN_PROGRESS_APPT
    ):
        href = wrapped_appointment.href
    else:
        href = "#"
    disabled = "disabled" if href == "#" else ""
    if view_appointment and wrapped_appointment.object.site.id == request.site.id:
        title = "" if disabled else "Edit appointment"
    else:
        title = "No permission to edit"
    return dict(
        wrapped_appointment=wrapped_appointment,
        view_appointment=view_appointment,
        href=href,
        disabled=disabled,
        title=title,
        request=request,
        IN_PROGRESS_APPT=IN_PROGRESS_APPT,
        anchor_id=anchor_id,
    )


@register.inclusion_tag(
    f"edc_subject_dashboard/bootstrap{get_bootstrap_version()}/"
    "dashboard/unscheduled_appointment_button.html"
)
def show_dashboard_unscheduled_appointment_button(
    wrapped_appointment=None, view_appointment=None, request=None
):
    show_button = False
    if (
        wrapped_appointment
        and wrapped_appointment.object.relative_next
        and wrapped_appointment.appt_status in [INCOMPLETE_APPT, COMPLETE_APPT]
        and (
            wrapped_appointment.object.appt_datetime + relativedelta(days=1)
            != wrapped_appointment.object.relative_next.appt_datetime
        )
        # and (
        #     wrapped_appointment.object.relative_next.visit_code
        #     != wrapped_appointment.visit_code
        # )
    ):
        show_button = True
    anchor_id = (
        f"unscheduled_appt_btn_{wrapped_appointment.visit_code }_"
        f"{ wrapped_appointment.visit_code_sequence }"
    )

    if view_appointment and wrapped_appointment.object.site.id == request.site.id:
        href = wrapped_appointment.unscheduled_appointment_url
    else:
        href = "#"
    disabled = "disabled" if href == "#" else ""
    if view_appointment and wrapped_appointment.object.site.id == request.site.id:
        title = "" if disabled else "Edit appointment"
    else:
        title = "No permission to edit"
    return dict(
        show_button=show_button,
        anchor_id=anchor_id,
        wrapped_appointment=wrapped_appointment,
        view_appointment=view_appointment,
        href=href,
        disabled=disabled,
        title=title,
        request=request,
        INCOMPLETE_APPT=INCOMPLETE_APPT,
        COMPLETE_APPT=COMPLETE_APPT,
    )



from django.core.exceptions import ObjectDoesNotExist
from django.views.generic.base import ContextMixin
from edc_visit_schedule.models import VisitSchedule


class SubjectVisitViewMixinError(Exception):
    pass


class SubjectVisitViewMixin(ContextMixin):

    """Mixin to add the subject visit instance to the view.

    Declare together with the edc_appointment.AppointmentViewMixin.
    """

    visit_attr = "subjectvisit"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.subject_visit = None

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        try:
            appointment = self.appointment
        except AttributeError as e:
            raise SubjectVisitViewMixinError(
                f"Mixin must be declared together with AppointmentViewMixin. Got {e}"
            )
        else:
            if appointment:
                try:
                    self.subject_visit = appointment.related_visit_model_attr
                except AttributeError as e:
                    raise SubjectVisitViewMixinError(
                        f"Visit model must have a OneToOne relation to appointment. "
                        f"Got {e}"
                    )
                else:
                    try:
                        self.subject_visit = appointment.visit
                    except AttributeError:
                        pass
                context.update(
                    subject_visit=self.subject_visit,
                    visit_schedule_pk=str(self.get_visit_schedule_pk(appointment)),
                )

        return context

    @staticmethod
    def get_visit_schedule_pk(appointment):
        """Returns a str(pk) from the VisitSchedule model."""
        visit_schedule_pk = ""
        opts = dict(
            visit_schedule_name=appointment.visit_schedule_name,
            schedule_name=appointment.schedule_name,
            visit_code=appointment.visit_code,
        )
        try:
            obj = VisitSchedule.objects.get(**opts)
        except ObjectDoesNotExist:
            pass
        else:
            visit_schedule_pk = str(obj.pk)
        return visit_schedule_pk



from dateutil.relativedelta import relativedelta
from django.test import TestCase
from edc_facility import import_holidays
from edc_locator.view_mixins import (
    SubjectLocatorViewMixin,
    SubjectLocatorViewMixinError,
)
from edc_registration.models import RegisteredSubject
from edc_utils import get_utcnow
from edc_visit_schedule import site_visit_schedules

from edc_subject_dashboard.view_mixins import (
    SubjectVisitViewMixin,
    SubjectVisitViewMixinError,
)

from ..models import (
    Appointment,
    BadSubjectVisit,
    OnScheduleOne,
    SubjectVisit,
    TestModel,
)
from ..visit_schedule import visit_schedule1


class DummyModelWrapper:
    def __init__(self, **kwargs):
        pass


class TestViewMixins(TestCase):
    def setUp(self):
        site_visit_schedules._registry = {}
        site_visit_schedules.register(visit_schedule=visit_schedule1)

        self.subject_identifier = "12345"

        onschedule_datetime = get_utcnow() - relativedelta(years=1)

        RegisteredSubject.objects.create(
            subject_identifier=self.subject_identifier,
            registration_datetime=onschedule_datetime,
        )

        OnScheduleOne.objects.create(
            subject_identifier=self.subject_identifier,
            onschedule_datetime=onschedule_datetime,
        )

        self.appointment = Appointment.objects.get(visit_code="1000")
        self.subject_visit = SubjectVisit.objects.create(
            appointment=self.appointment,
            subject_identifier=self.subject_identifier,
            visit_schedule_name="visit_schedule1",
            schedule_name="schedule1",
            visit_code="1000",
        )
        self.bad_subject_visit = BadSubjectVisit.objects.create(
            appointment=self.appointment, subject_identifier=self.subject_identifier
        )
        self.test_model = TestModel.objects.create(subject_visit=self.subject_visit)

    @classmethod
    def setUpTestData(cls):
        import_holidays()

    def test_subject_visit_missing_appointment(self):
        mixin = SubjectVisitViewMixin()
        self.assertRaises(SubjectVisitViewMixinError, mixin.get_context_data)

    def test_subject_visit_correct_relation(self):
        mixin = SubjectVisitViewMixin()
        mixin.appointment = self.appointment
        context = mixin.get_context_data()
        self.assertEqual(context.get("subject_visit"), self.subject_visit)

    def test_subject_visit_incorrect_relation(self):
        """Asserts raises if relation is not one to one."""

        class MySubjectVisitViewMixin(SubjectVisitViewMixin):
            visit_attr = "badsubjectvisit"

        mixin = MySubjectVisitViewMixin()
        self.assertRaises(SubjectVisitViewMixinError, mixin.get_context_data)

    def test_subject_locator_raises_on_bad_model(self):
        class MySubjectLocatorViewMixin(SubjectLocatorViewMixin):
            subject_locator_model_wrapper_cls = DummyModelWrapper
            subject_locator_model = "blah.blahblah"

        mixin = MySubjectLocatorViewMixin()
        self.assertRaises(SubjectLocatorViewMixinError, mixin.get_context_data)

    def test_subject_locator_raisesmissing_wrapper_cls(self):
        class MySubjectLocatorViewMixin(SubjectLocatorViewMixin):
            subject_locator_model = "edc_locator.subjectlocator"

        self.assertRaises(SubjectLocatorViewMixinError, MySubjectLocatorViewMixin)

    def test_subject_locator_ok(self):
        class MySubjectLocatorViewMixin(SubjectLocatorViewMixin):
            subject_locator_model_wrapper_cls = DummyModelWrapper
            subject_locator_model = "edc_locator.subjectlocator"

        mixin = MySubjectLocatorViewMixin()
        mixin.kwargs = {"subject_identifier": self.subject_identifier}
        try:
            mixin.get_context_data()
        except SubjectLocatorViewMixinError as e:
            self.fail(e)



from django.conf import settings
from edc_appointment.constants import (
    COMPLETE_APPT,
    IN_PROGRESS_APPT,
    INCOMPLETE_APPT,
    NEW_APPT,
)
from edc_constants.constants import (
    COMPLETE,
    INCOMPLETE,
    NO,
    NOT_APPLICABLE,
    UNKNOWN,
    YES,
)
from edc_dashboard import insert_bootstrap_version, url_names
from edc_visit_tracking.constants import MISSED_VISIT

from .dashboard_templates import dashboard_templates


class DashboardMiddleware:

    """Declare in settings:

    For example:

        DASHBOARD_URL_NAMES = {
            'subject_models_url': 'subject_models_url',
            'subject_listboard_url': 'myapp:subject_listboard_url',
            'subject_dashboard_url': 'myapp:subject_dashboard_url',
        }

        DASHBOARD_BASE_TEMPLATES = {
            'listboard_base_template': 'myapp/base.html',
            'dashboard_base_template': 'myapp/base.html',
            'subject_listboard_template': 'myapp/listboard.html',
            'subject_dashboard_template': 'myapp/dashboard.html',
        }


    """

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        return response

    def process_view(self, request, *args):
        """Adds/Updates references to urls and templates."""

        try:
            settings.DASHBOARD_URL_NAMES
        except AttributeError:
            pass
        else:
            url_names.register_from_dict(**settings.DASHBOARD_URL_NAMES)
        request.url_name_data = url_names.registry

        try:
            template_data = settings.DASHBOARD_BASE_TEMPLATES
        except AttributeError:
            template_data = dashboard_templates
        template_data = insert_bootstrap_version(**template_data)
        request.template_data.update(**template_data)

    def process_template_response(self, request, response):
        if response.context_data:
            response.context_data.update(**request.url_name_data)
            response.context_data.update(**request.template_data)
            response.context_data.update(
                COMPLETE_APPT=COMPLETE_APPT,
                INCOMPLETE_APPT=INCOMPLETE_APPT,
                IN_PROGRESS_APPT=IN_PROGRESS_APPT,
                MISSED_VISIT=MISSED_VISIT,
                NEW_APPT=NEW_APPT,
                INCOMPLETE=INCOMPLETE,
                COMPLETE=COMPLETE,
                YES=YES,
                NO=NO,
                NOT_APPLICABLE=NOT_APPLICABLE,
                UNKNOWN=UNKNOWN,
            )
        return response



from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.messages import SUCCESS
from django.http.response import HttpResponseRedirect
from django.urls import reverse
from django.views import View
from edc_dashboard import url_names
from edc_utils import get_utcnow
from edc_visit_tracking.utils import get_subject_visit_model_cls


class RefreshMetadataActionsView(LoginRequiredMixin, View):
    @staticmethod
    def refresh_metadata(subject_visit_id=None, **kwargs):  # noqa
        subject_visit = get_subject_visit_model_cls().objects.get(id=subject_visit_id)
        subject_visit.save()
        return subject_visit

    def get(self, request, *args, **kwargs):
        dte1 = get_utcnow()
        subject_visit = self.refresh_metadata(**kwargs)

        url_name = url_names.get("subject_dashboard_url")
        args = (
            subject_visit.appointment.subject_identifier,
            str(subject_visit.appointment.id),
        )
        url = reverse(url_name, args=args)
        messages.add_message(
            request,
            SUCCESS,
            f"The data collection schedule for {subject_visit.visit_code}."
            f"{subject_visit.visit_code_sequence} has been refreshed "
            f"({round((get_utcnow()-dte1).microseconds/1000000, 2)} seconds)",
        )
        return HttpResponseRedirect(url)

    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def head(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def options(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def delete(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def put(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def patch(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)
